@@ -1,35 +1,43 @@
 # coding=utf-8
 # Copyright (C) 2019  Luis Felipe Mileo - KMEE
 
-from __future__ import division, print_function, unicode_literals
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
 
-import re
+import collections
 import datetime
 import time
-from lxml import etree
-import collections
 
-from nfelib.v4_00 import leiauteNFe
-from nfelib.v4_00 import leiauteNFe_sub as nfe_sub
-from nfelib.v4_00 import consStatServ
-from nfelib.v4_00 import retConsStatServ
-from nfelib.v4_00 import consSitNFe
-from nfelib.v4_00 import retConsSitNFe
-from nfelib.v4_00 import enviNFe
-from nfelib.v4_00 import retEnviNFe
-from nfelib.v4_00 import consReciNFe
-from nfelib.v4_00 import retConsReciNFe
-from nfelib.v4_00 import leiauteEvento
-from nfelib.v4_00 import leiauteEventoCancNFe
-from nfelib.v4_00 import leiauteCCe
-from nfelib.v4_00 import retEnvEvento
-from nfelib.v4_00 import leiauteInutNFe
+from lxml import etree
+
 from erpbrasil.edoc.edoc import DocumentoEletronico
 
 try:
-    from StringIO import StringIO
+    # nfelib imports
+    # xsd NFe
+    from nfelib.v4_00 import retInutNFe
+    from nfelib.v4_00 import retConsStatServ
+    from nfelib.v4_00 import retConsSitNFe
+    from nfelib.v4_00 import retEnviNFe
+    from nfelib.v4_00 import retConsReciNFe
+
+    # xsd Distribuiçao NFe
+    from nfelib.v4_00 import distDFeInt
+    from nfelib.v4_00 import retDistDFeInt
+
+    # xsd Evento Generico
+    from nfelib.v4_00 import retEnvEvento
+
+    # xsd Evento Cancelamento
+    from nfelib.v4_00 import retEnvEventoCancNFe
+
+    # xsd CCe
+    from nfelib.v4_00 import retEnvCCe
+
 except ImportError:
-    from io import StringIO
+    pass
+
 
 TEXTO_CARTA_CORRECAO = """A Carta de Correcao e disciplinada pelo paragrafo \
 1o-A do art. 7o do Convenio S/N, de 15 de dezembro de 1970 e \
@@ -42,19 +50,19 @@ a correcao de dados cadastrais que implique mudanca do \
 remetente ou do destinatario; III - a data de emissao \
 ou de saida."""
 
-WS_NFE_INUTILIZACAO = 'NfeInutilizacao'
-WS_NFE_CONSULTA = 'NfeConsultaProtocolo'
-WS_NFE_SITUACAO = 'NfeStatusServico'
-WS_NFE_RECEPCAO_EVENTO = 'RecepcaoEvento'
-WS_NFE_AUTORIZACAO = 'NfeAutorizacao'
-WS_NFE_RET_AUTORIZACAO = 'NfeRetAutorizacao'
-
-WS_NFE_CADASTRO = 'NfeConsultaCadastro'
-
-WS_NFCE_QR_CODE = 'NfeQRCode'
-WS_NFCE_CONSULTA_DESTINADAS = 'NfeConsultaDest'
 WS_DFE_DISTRIBUICAO = 'NFeDistribuicaoDFe'
 WS_DOWNLOAD_NFE = 'nfeDistDFeInteresse'
+WS_NFCE_CONSULTA_DESTINADAS = 'NfeConsultaDest'
+WS_NFCE_QR_CODE = 'NfeQRCode'
+WS_NFE_AUTORIZACAO = 'NfeAutorizacao'
+WS_NFE_CADASTRO = 'NfeConsultaCadastro'
+
+WS_NFE_CONSULTA = 'NfeConsultaProtocolo'
+WS_NFE_INUTILIZACAO = 'NfeInutilizacao'
+WS_NFE_RECEPCAO_EVENTO = 'RecepcaoEvento'
+WS_NFE_RET_AUTORIZACAO = 'NfeRetAutorizacao'
+
+WS_NFE_SITUACAO = 'NfeStatusServico'
 
 AMBIENTE_PRODUCAO = 1
 AMBIENTE_HOMOLOGACAO = 2
@@ -144,7 +152,7 @@ SVAN = {
     AMBIENTE_PRODUCAO: {
         'servidor': 'www.sefazvirtual.fazenda.gov.br',
         WS_NFE_INUTILIZACAO: 'NFeInutilizacao4/NFeInutilizacao4.asmx?wsdl',
-        WS_NFE_CONSULTA: 'NFeConsultaProtocolo4/NFeConsultaProtocolo4.asmx?wsdl',   # noqa
+        WS_NFE_CONSULTA: 'NFeConsultaProtocolo4/NFeConsultaProtocolo4.asmx?wsdl',  # noqa
         WS_NFE_SITUACAO: 'NFeStatusServico4/NFeStatusServico4.asmx?wsdl',
         WS_NFE_RECEPCAO_EVENTO: 'NFeRecepcaoEvento4/NFeRecepcaoEvento4.asmx?wsdl',  # noqa
         WS_NFE_AUTORIZACAO: 'NFeAutorizacao4/NFeAutorizacao4.asmx?wsdl',
@@ -330,7 +338,6 @@ UFCE = {
     }
 }
 
-
 UFGO = {
     AMBIENTE_PRODUCAO: {
         'servidor': 'nfe.sefaz.go.gov.br',
@@ -354,7 +361,6 @@ UFGO = {
     }
 }
 
-
 UFMT = {
     NFE_MODELO: {
         AMBIENTE_PRODUCAO: {
@@ -547,7 +553,6 @@ UFPE = {
     }
 }
 
-
 UFRS = {
     NFE_MODELO: {
         AMBIENTE_PRODUCAO: {
@@ -565,7 +570,7 @@ UFRS = {
             WS_NFE_INUTILIZACAO: 'ws/nfeinutilizacao/nfeinutilizacao4.asmx?wsdl',  # noqa
             WS_NFE_CONSULTA: 'ws/NfeConsulta/NfeConsulta4.asmx?wsdl',
             WS_NFE_SITUACAO: 'ws/NfeStatusServico/NfeStatusServico4.asmx?wsdl',
-            WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento4.asmx?wsdl',  #noqa
+            WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento4.asmx?wsdl',  # noqa
             WS_NFE_AUTORIZACAO: 'ws/NfeAutorizacao/NFeAutorizacao4.asmx?wsdl',
             WS_NFE_RET_AUTORIZACAO: 'ws/NfeRetAutorizacao/NFeRetAutorizacao4.asmx?wsdl',  # noqa
             WS_NFE_CADASTRO: 'ws/cadconsultacadastro/cadconsultacadastro4.asmx?wsdl',  # noqa
@@ -577,7 +582,7 @@ UFRS = {
             WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento.asmx',
             WS_NFE_AUTORIZACAO: 'ws/NfeAutorizacao/NFeAutorizacao.asmx',
             WS_NFE_RET_AUTORIZACAO: 'ws/NfeRetAutorizacao/NFeRetAutorizacao.asmx',  # noqa
-            WS_NFE_CADASTRO: 'ws/cadconsultacadastro/cadconsultacadastro2.asmx',   # noqa
+            WS_NFE_CADASTRO: 'ws/cadconsultacadastro/cadconsultacadastro2.asmx',  # noqa
             WS_NFE_INUTILIZACAO: 'ws/NfeInutilizacao/NfeInutilizacao2.asmx',
             WS_NFE_CONSULTA: 'ws/NfeConsulta/NfeConsulta2.asmx',
             WS_NFE_SITUACAO: 'ws/NfeStatusServico/NfeStatusServico2.asmx',
@@ -588,7 +593,7 @@ UFRS = {
             WS_NFE_RECEPCAO_EVENTO: 'ws/recepcaoevento/recepcaoevento.asmx',
             WS_NFE_AUTORIZACAO: 'ws/NfeAutorizacao/NFeAutorizacao.asmx',
             WS_NFE_RET_AUTORIZACAO: 'ws/NfeRetAutorizacao/NFeRetAutorizacao.asmx',  # noqa
-            WS_NFE_CADASTRO: 'ws/cadconsultacadastro/cadconsultacadastro2.asmx',   # noqa
+            WS_NFE_CADASTRO: 'ws/cadconsultacadastro/cadconsultacadastro2.asmx',  # noqa
             WS_NFE_INUTILIZACAO: 'ws/NfeInutilizacao/NfeInutilizacao2.asmx',
             WS_NFE_CONSULTA: 'ws/NfeConsulta/NfeConsulta2.asmx',
             WS_NFE_SITUACAO: 'ws/NfeStatusServico/NfeStatusServico2.asmx',
@@ -646,7 +651,6 @@ UFSP = {
     }
 }
 
-
 ESTADO_WS = {
     'AC': SVRS,
     'AL': SVRS,
@@ -678,6 +682,7 @@ ESTADO_WS = {
     'AN': AN,
 }
 
+
 def localizar_url(servico, estado, mod='55', ambiente=2):
     sigla = SIGLA_ESTADO[estado]
     ws = ESTADO_WS[sigla]
@@ -694,14 +699,14 @@ def localizar_url(servico, estado, mod='55', ambiente=2):
 
     if sigla == 'RS' and servico == WS_NFE_CADASTRO:
         dominio = 'cad.sefazrs.rs.gov.br'
-    if sigla in ('AC', 'RN', 'PB', 'SC', 'RJ') and \
-       servico == WS_NFE_CADASTRO:
+    if sigla in ('AC', 'RN', 'PB', 'SC', 'RJ') and servico == WS_NFE_CADASTRO:
         dominio = 'cad.svrs.rs.gov.br'
     if sigla == 'AN' and servico == WS_NFE_RECEPCAO_EVENTO:
         dominio = 'www.nfe.fazenda.gov.br'
 
     return "https://%s/%s" % (dominio, complemento)
 
+
 Metodo = collections.namedtuple('Metodo', ['webservice', 'metodo'])
 
 METODO_WS = {
@@ -741,7 +746,7 @@ class NFe(DocumentoEletronico):
         return edoc.infNFe.Id[:3], edoc.infNFe.Id[3:]
 
     def status_servico(self):
-        raiz = consStatServ.TConsStatServ(
+        raiz = retConsStatServ.TConsStatServ(
             versao=self.versao,
             tpAmb=self.ambiente,
             cUF=self.uf,
@@ -758,7 +763,7 @@ class NFe(DocumentoEletronico):
         )
 
     def consulta_documento(self, chave):
-        raiz = consSitNFe.TConsSitNFe(
+        raiz = retConsSitNFe.TConsSitNFe(
             versao=self.versao,
             tpAmb=self.ambiente,
             xServ='CONSULTAR',
@@ -786,7 +791,7 @@ class NFe(DocumentoEletronico):
         """
         xml_assinado = self.assina_raiz(edoc, edoc.infNFe.Id)
 
-        raiz = enviNFe.TEnviNFe(
+        raiz = retEnviNFe.TEnviNFe(
             versao=self.versao,
             idLote=datetime.datetime.now().strftime('%Y%m%d%H%M%S'),
             indSinc='0'
@@ -809,7 +814,7 @@ class NFe(DocumentoEletronico):
         )
 
     def envia_inutilizacao(self, evento):
-        tinut = leiauteInutNFe.TInutNFe(
+        tinut = retInutNFe.TInutNFe(
             versao=self.versao,
             infInut=evento,
             Signature=None)
@@ -824,7 +829,7 @@ class NFe(DocumentoEletronico):
             localizar_url(WS_NFE_INUTILIZACAO, str(self.uf), self.mod,
                           int(self.ambiente)),
             'nfeInutilizacaoNF',
-            leiauteInutNFe
+            retInutNFe
         )
 
     def consulta_recibo(self, numero=False, proc_envio=False):
@@ -834,7 +839,7 @@ class NFe(DocumentoEletronico):
         if not numero:
             return
 
-        raiz = consReciNFe.TConsReciNFe(
+        raiz = retConsReciNFe.TConsReciNFe(
             versao=self.versao,
             tpAmb=self.ambiente,
             nRec=numero,
@@ -853,14 +858,14 @@ class NFe(DocumentoEletronico):
         if not numero_lote:
             numero_lote = self._gera_numero_lote()
 
-        raiz = leiauteEvento.TEnvEvento(versao="1.00", idLote=numero_lote)
+        raiz = retEnvEvento.TEnvEvento(versao="1.00", idLote=numero_lote)
         raiz.original_tagname_ = 'envEvento'
         xml_envio_string, xml_envio_etree = self._generateds_to_string_etree(
             raiz
         )
 
         for raiz_evento in lista_eventos:
-            evento = leiauteEventoCancNFe.TEvento(
+            evento = retEnvEventoCancNFe.TEvento(
                 versao="1.00", infEvento=raiz_evento,
             )
             evento.original_tagname_ = 'evento'
@@ -879,7 +884,7 @@ class NFe(DocumentoEletronico):
                           data_hora_evento=False):
         tipo_evento = '110111'
         sequencia = '1'
-        raiz = leiauteEventoCancNFe.infEventoType(
+        raiz = retEnvEventoCancNFe.infEventoType(
             Id='ID' + tipo_evento + chave + sequencia.zfill(2),
             cOrgao=self.uf,
             tpAmb=self.ambiente,
@@ -889,7 +894,7 @@ class NFe(DocumentoEletronico):
             tpEvento='110111',
             nSeqEvento='1',
             verEvento='1.00',
-            detEvento=leiauteEventoCancNFe.detEventoType(
+            detEvento=retEnvEventoCancNFe.detEventoType(
                 versao="1.00",
                 descEvento='Cancelamento',
                 nProt=protocolo_autorizacao,
@@ -902,18 +907,18 @@ class NFe(DocumentoEletronico):
     def carta_correcao(self, chave, sequencia, justificativa,
                        data_hora_evento=False):
         tipo_evento = '110110'
-        raiz = leiauteCCe.infEventoType(
+        raiz = retEnvCCe.infEventoType(
             Id='ID' + tipo_evento + chave + sequencia.zfill(2),
             cOrgao=self.uf,
             tpAmb=self.ambiente,
             CNPJ=chave[6:20],
-            # CPF=None,
+            CPF=None,
             chNFe=chave,
             dhEvento=data_hora_evento or self._hora_agora(),
             tpEvento=tipo_evento,
             nSeqEvento=sequencia,
             verEvento='1.00',
-            detEvento=leiauteCCe.detEventoType(
+            detEvento=retEnvCCe.detEventoType(
                 versao="1.00",
                 descEvento='Carta de Correcao',
                 xCorrecao=justificativa,
@@ -927,7 +932,7 @@ class NFe(DocumentoEletronico):
                      justificativa):
         ano = str(datetime.date.today().year)[2:]
         uf = str(self.uf)
-        raiz = leiauteInutNFe.infInutType(
+        raiz = retInutNFe.infInutType(
             Id='ID' + uf + ano + cnpj + mod + serie.zfill(3) +
                str(num_ini).zfill(9) + str(num_fin).zfill(9),
             tpAmb=self.ambiente,
@@ -945,8 +950,7 @@ class NFe(DocumentoEletronico):
         return raiz
 
     def _verifica_servico_em_operacao(self, proc_servico):
-        if proc_servico.resposta.cStat == \
-                self._edoc_situacao_servico_em_operacao:
+        if proc_servico.resposta.cStat == self._edoc_situacao_servico_em_operacao:
             return True
         return False
 
@@ -968,3 +972,56 @@ class NFe(DocumentoEletronico):
         if proc_recibo.resposta.cStat == '105':
             return True
         return False
+
+    def consultar_distribuicao(self, cnpj_cpf, ultimo_nsu=False,
+                               nsu_especifico=False, chave=False):
+        """
+
+        :param cnpj_cpf: CPF ou CNPJ a ser consultado
+        :param ultimo_nsu: Último NSU para pesquisa. Formato: '999999999999999'
+        :param nsu_especifico: NSU Específico para pesquisa.
+                                Formato: '999999999999999'
+        :param chave: Chave de acesso do documento
+        :return: Retorna uma estrutura contendo as estruturas de envio
+        e retorno preenchidas
+        """
+
+        if not ultimo_nsu and not nsu_especifico and not chave:
+            return
+
+        distNSU = consNSU = consChNFe = None
+        if ultimo_nsu:
+            distNSU = distDFeInt.distNSUType(
+                ultNSU=ultimo_nsu
+            )
+        if nsu_especifico:
+            consNSU = distDFeInt.consNSUType(
+                NSU=nsu_especifico
+            )
+        if chave:
+            consChNFe = distDFeInt.consChNFeType(
+                chNFe=chave
+            )
+
+        if distNSU and consNSU or distNSU and consChNFe or consNSU and consChNFe:
+            # TODO: Raise?
+            return
+
+        raiz = distDFeInt.distDFeInt(
+            versao=self.versao,
+            tpAmb=self.ambiente,
+            cUFAutor=self.uf,
+            CNPJ=cnpj_cpf if len(cnpj_cpf) > 11 else None,
+            CPF=cnpj_cpf if len(cnpj_cpf) <= 11 else None,
+            distNSU=distNSU,
+            consNSU=consNSU,
+            consChNFe=consChNFe,
+        )
+
+        return self._post(
+            raiz,
+            localizar_url(WS_DFE_DISTRIBUICAO, str(self.uf), self.mod,
+                          int(self.ambiente)),
+            'nfeDistDFeInteresse',
+            retDistDFeInt
+        )
